@@ -1,6 +1,5 @@
 import datetime
 import os
-import re
 
 import git
 import semver
@@ -107,27 +106,9 @@ class KACLDocument:
         # 3. assert versions in valid format
         versions = self.versions()
         for v in versions:
-            if "Unreleased" != v.version():
-                raw = v.raw()
-                regex = KACLParser.semver_regex
-                regex_error = r"#\s+(.*)\s+"
-                if v.link():
-                    regex = f"#\\s+\\[{KACLParser.semver_regex}\\]"
-                    regex_error = r"#\s+\[(.*)\]"
-                if not KACLParser.parse_sem_ver(raw, regex):
-                    start_pos = 0
-                    end_pos = 0
-                    m = re.match(regex_error, raw)
-                    if m:
-                        start_pos = raw.find(m.group(1))
-                        end_pos = start_pos + len(m.group(1))
-                    validation.add_error(
-                        line=raw,
-                        line_number=v.line_number(),
-                        error_message="Version is not a valid semantic version.",
-                        start_character_pos=start_pos,
-                        end_character_pos=end_pos,
-                    )
+            validation_errors = v.validate(self.config)
+            for e in validation_errors:
+                validation.add(e)
 
         # 3.1 assert versions in descending order
         for i in range(len(versions) - 1):
@@ -145,80 +126,11 @@ class KACLDocument:
             except Exception:
                 pass
 
-        # 3.2 assert versions have a valid date
-        for v in versions:
-            if "Unreleased" != v.version():
-                if not v.date() or len(v.date()) < 1:
-                    validation.add_error(
-                        line=v.raw(),
-                        line_number=v.line_number(),
-                        error_message="Versions need to be decorated with a release date in the following format 'YYYY-MM-DD'",
-                        start_character_pos=0,
-                        end_character_pos=len(v.raw()),
-                    )
-                if v.date() and not re.match(r"\d\d\d\d-[0-1][\d]-[0-3][\d]", v.date()):
-                    start_pos = v.raw().find(v.date())
-                    end_pos = start_pos + len(v.date())
-                    validation.add_error(
-                        line=v.raw(),
-                        line_number=v.line_number(),
-                        error_message="Date does not match format 'YYYY-MM-DD'",
-                        start_character_pos=start_pos,
-                        end_character_pos=end_pos,
-                    )
-
-            # 3.3 check that only allowed sections are in the version
-            sections = v.sections()
-            for title, element in sections.items():
-                if title not in self.config.allowed_version_sections:
-                    start_pos = element.raw().find(title)
-                    end_pos = start_pos + len(title)
-                    validation.add_error(
-                        line=element.raw(),
-                        line_number=element.line_number(),
-                        error_message=f'"{title}" is not a valid section for a version. Options are [{",".join( self.config.allowed_version_sections)}]',
-                        start_character_pos=start_pos,
-                        end_character_pos=end_pos,
-                    )
-                # 3.4 check that only list elements are in the sections
-                # 3.4.1 bring everything into a single line
-                body = element.body()
-                body_clean = re.sub(r"\n\s+", "", body)
-                lines = body_clean.split("\n\n")
-                non_list_lines = [
-                    x
-                    for x in lines
-                    if not x.strip().startswith("-") and len(x.strip()) > 0
-                ]
-                if len(non_list_lines) > 0:
-                    validation.add_error(
-                        line=body.strip(),
-                        line_number=element.line_number(),
-                        error_message="Section does contain more than only listings.",
-                    )
-            # 3.5 make sure that every version that has content has it's content in a section
-            if len(v.sections()) == 0 and len(v.body().strip()) != 0:
-                validation.add_error(
-                    line=v.raw(),
-                    line_number=v.line_number(),
-                    error_message=f'Version "{v.version()}" has change elements outside of a change section.',
-                )
-
-            # 3.6 Check that a link exists for linked versions
-            if "[" in v.raw() and "]" in v.raw() and not v.has_link_reference():
-                validation.add_error(
-                    line=v.raw(),
-                    line_number=v.line_number(),
-                    error_message=f'Version "{v.version()}" is linked, but no link reference found in changelog file.',
-                    start_character_pos=v.raw().find("["),
-                    end_character_pos=v.raw().find("]"),
-                )
-
         # 4 link references
         # 4.1 check that there are only linked references
-        version_strings = [v.version() for v in versions]
+        version_strings = [v.version().lower() for v in versions]
         for v, link in self.__link_references.items():
-            if v not in version_strings:
+            if v.lower() not in version_strings:
                 validation.add_error(
                     line=link.raw(),
                     line_number=link.line_number(),
@@ -237,6 +149,49 @@ class KACLDocument:
         validation_results = self.validate()
         return validation_results.is_valid()
 
+    def squash(self, version_start, version_end, keep_version_info=True):
+
+        if isinstance(version_start, str):
+            version_start = semver.VersionInfo.parse(version_start)
+        if isinstance(version_end, str):
+            version_end = semver.VersionInfo.parse(version_end)
+
+        versions = self.versions()
+        versions.sort(reverse=True)
+        versions_to_squash = []
+        versions_unsquashed = []
+        for v in versions:
+            if "unreleased" in v.version().lower():
+                versions_unsquashed.append(v)
+            elif v.semver() >= version_start and v.semver() <= version_end:
+                versions_to_squash.append(v)
+            else:
+                versions_unsquashed.append(v)
+
+        # sort versions descending
+        versions_to_squash.sort(reverse=True)
+
+        # get the version that will be the new version
+        squash_version = versions_to_squash[0]
+
+        # remove version from the squashed list
+        versions_to_squash.remove(squash_version)
+
+        # add all sections from the squashed versions to the new version
+        for v in versions_to_squash:
+            for section, changes in v.sections().items():
+                for change in changes.items():
+                    if keep_version_info:
+                        if v.has_link_reference():
+                            change = f"[[{v.version()}]({v.link()})] {change}"
+                        else:
+                            change = f"[{v.version()}] {change}"
+                    squash_version.add(section, change)
+
+        versions_unsquashed.append(squash_version)
+        versions_unsquashed.sort(reverse=True)
+        self.__versions = versions_unsquashed
+
     def has_changes(self):
         unreleased_version = self.get("Unreleased")
         if not unreleased_version:
@@ -545,7 +500,7 @@ The format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),
 
         # set link references into versions if available
         for v in versions:
-            v.set_link(link_references.get(v.version(), None))
+            v.set_link(link_references.get(v.version().lower(), None))
 
         return KACLDocument(
             data=data,
